﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Http;
using System.Web.Optimization;

namespace CanLlistes
{
    public class Global : System.Web.HttpApplication
    {
        /// <summary>
        /// per propòsits de desenvolupament, testejar concurrencia en una màquina sola i d'altres
        /// </summary>
        private bool developing = true;

        #region events de sessio i aplicacio del IIS

        protected void Application_Start(object sender, EventArgs e)
        {
            BundleConfig.RegisterBundles(BundleTable.Bundles);
            WebApiConfig.Register(GlobalConfiguration.Configuration);
            
        }
        protected void Session_Start(object sender, EventArgs e)
        {
            controlSessio nou = nouObjecteDeSessioActual();
            
            if (!developing) { mataAltresSessionsDeIp(nou.IpRemota, nou.IDsessio); }

            Application[nou.IDsessio] = nou;
            
        }
        protected void Session_End(object sender, EventArgs e)
        {
            Application.Remove(Session.SessionID);
        }
        protected void Application_BeginRequest(object sender, EventArgs e)
        {

        }
        protected void Application_AuthenticateRequest(object sender, EventArgs e)
        {

        }
        protected void Application_Error(object sender, EventArgs e)
        {

        }
        protected void Application_End(object sender, EventArgs e)
        {

        }

        #endregion


        #region funcions de control
        /// <summary>
        /// sol·licitud de renovació d'objecte de sessió en servidor
        /// </summary>
        public void renovaSessio()
        {
            controlSessio nou = nouObjecteDeSessioActual();

            if (!developing) { mataAltresSessionsDeIp(nou.IpRemota, nou.IDsessio); }

            Application[nou.IDsessio] = nou;

        }
        /// <summary>
        /// comprova si algú esta executant el flux d'un xml, cal per garantir una execució per client
        /// </summary>
        /// <param name="arxiuXML">el nom del xml que es cerca en els objectes de sessió</param>
        /// <remarks> Cerca en la cadena xmlExecutant de cada sessio.
        /// </remarks>
        /// <returns> true si algu esta executant
        /// </returns>
        public bool alguExecuta(string arxiuXML)
        {
            // ha de rebre XML
            bool trobat = false;
            controlSessio sessio;

            foreach (var sessioDesada in Application)
            {
                sessio = (controlSessio)Application.Get((string)sessioDesada);
                if (!string.IsNullOrEmpty(sessio.xmlExecutant))
                {
                    if (arxiuXML.Contains(sessio.xmlExecutant)) trobat = true;
                }
            }
            return trobat;
        }
        /// <summary>
        /// comprova si algú esta editant un xml, per nom de correu destinació
        /// </summary>
        /// <param name="arxiuXML">arxiu xml a cercar en sessions</param>
        /// <param name="sessioQueHoDemana">hash de la sessió que ho demana perque no es detecti a si mateixa</param>
        /// <returns>true si algu edita</returns>
        public bool alguEdita(string arxiuXML, string sessioQueHoDemana = "")
        {
            bool trobat = false;
            controlSessio sessio;
            foreach (var sessioDesada in Application)
            {
                sessio = (controlSessio)Application.Get((string)sessioDesada);
                if (sessio.llistaDeLaSessio != null)
                {
                    if (sessio.llistaDeLaSessio.mailDesti.ToUpper().Equals(arxiuXML.ToUpper().Substring(0, arxiuXML.Length - 4)) && (!sessio.IDsessio.Equals(sessioQueHoDemana))) trobat = true;
                }

            }
            return trobat;
        }

        /// <summary>
        /// elimina del servidor la sessió de la que es dona el hash
        /// </summary>
        /// <param name="sessioAMatar">hash de sessió a matar</param>
        /// <returns>true si tot ha anat be</returns>
        public bool mataSessio(string sessioAMatar)
        {

            controlSessio aquesta = new controlSessio();
            if (aquesta != null)
            {
                if (aquesta.xmlExecutant != "") aquesta.despresScriptAsincron();
                return true;
            }
            return false;

        }
        /// <summary>
        /// comprova si cap sessió es potencialment manipulador d'un xml donat
        /// </summary>
        /// <param name="XMLAmirar">nom de l'arxiu</param>
        /// <param name="sessioQueHoDemana">hash de sessió que ho demana per no trobar-se a ella</param>
        /// <returns>true si esta lliure</returns>
        public bool estalliureXML(string XMLAmirar, string sessioQueHoDemana = "")
        {
            return !(alguEdita(XMLAmirar, sessioQueHoDemana) || alguExecuta(XMLAmirar));
        }
        /// <summary>
        /// crea un objecte controlsessió nou segons la sessió actual
        /// </summary>
        /// <returns>objecte controlsessió</returns>
        private controlSessio nouObjecteDeSessioActual()
        {
            controlSessio nouObjecteSessio = new controlSessio();
            nouObjecteSessio.idioma = HttpContext.Current.Request.UserLanguages[0];
            nouObjecteSessio.llistaDeLaSessio = null;
            nouObjecteSessio.IpRemota = HttpContext.Current.Request.UserHostAddress;
            nouObjecteSessio.IDsessio = HttpContext.Current.Session.SessionID;
            return nouObjecteSessio;
        }
        /// <summary>
        /// elimina de memória les altres sessions d'una mateixa ip
        /// </summary>
        /// <param name="ip">ip de la que cercar sessions</param>
        /// <param name="sessioBona">la sessió que ho demana, no es borra a si mateixa</param>
        private void mataAltresSessionsDeIp(string ip, string sessioBona)
        {
            List<string> sessionsAMatar = new List<string>();
            controlSessio sessio;

            foreach (var sessioDesada in Application)
            {
                sessio = (controlSessio)Application.Get((string)sessioDesada);
                if (sessio.IpRemota.Equals(ip) && (!sessio.IDsessio.Equals(sessioBona)))
                {

                    if (sessio.xmlExecutant != "") sessio.despresScriptAsincron();
                    sessionsAMatar.Add(sessio.IDsessio);

                }
                if (sessio.IDsessio.Equals(sessioBona))
                {
                    //error , ja hi ha una sessio valida per aquesta ip amb el mateix nom?
                }
            }
            foreach (var sessioDesada in sessionsAMatar)
            {

                Application.Remove(sessioDesada);

            }

        }

        #endregion

    }
}